c-----------------------------------------------------------------------
c --- TRAJCRD -- Re-maps trajectory coordinates
c-----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01           Level: 090409              MAIN
c
c               D. Strimaitis, TRC Environmental Corp.
c
c     Copyright (c) 2009 by Exponent, Inc.
c
c --- PURPOSE: Processes one or more trajectory (.trc) files, converting
c              the coordinate pairs to a new map projection and DATUM.
c              The format of each output file is the same as the input
c              file.
c
c --- UPDATES:
c --- V1.0 (081223) to V1.01 (090409)  DGS
c       (1) COORDLIB updated to v1.99 
c
c --- TRAJCRD calls:     SETUP, COMP, FIN
c-----------------------------------------------------------------------
      Program TRAJCRD

c --- include QA common block
      include 'qa.trc'

c --- set version and level number of program
      ver='1.01'
      level='090409'

c --- Setup phase -- initialization & program setup operations
      call SETUP

c --- Computational phase -- basic time loop
      call COMP

c --- Termination phase -- program termination functions
      call FIN

      stop
      end

c----------------------------------------------------------------------
      BLOCK DATA
c----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01     Level: 081223              BLOCKDATA
c ---           D. Strimaitis
c
c----------------------------------------------------------------------
c --- Include parameter statements
      include 'params.trc'

c --- Include common blocks
      include 'ctldat.trc'
      include 'trajhd.trc'

c --- /CONTROL/ variables
      data ntrjdat/1/
      data lcfiles/.FALSE./
      data filelst/'TRAJCRD.LST'/
      data pmapt/'UTM     '/,datumt/'WGS-84  '/,utmhemt/'N   '/
      data feastt/0.0/,fnortht/0.0/
      data clat0t/'0.0N            '/,clon0t/'0.0E            '/
      data clat1t/'0.0N            '/,clat2t/'0.0N            '/

c --- /TRAJHD/ variables
      data pmap/'UTM     '/,datum/'WGS-84  '/,utmhem/'N   '/
      data feast/0.0/,fnorth/0.0/
      data clat0/'0.0N            '/,clon0/'0.0E            '/
      data clat1/'0.0N            '/,clat2/'0.0N            '/

      end

c----------------------------------------------------------------------
c --- Include subroutine packages from CALPUFF
      include 'calutils.subs'

c --- Include coordinate transformation package
      include 'coordlib.for'
c----------------------------------------------------------------------


c----------------------------------------------------------------------
      subroutine setup
c----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01     Level: 081223                  SETUP
c ---           D. Strimaitis
c
c --- PURPOSE:  Coordinate program set-up operations
c
c --- INPUTS:
c
c --- OUTPUT:
c
c --- SETUP called by: (main)
c
c --- SETUP calls:      DATETM, COMLINE, RDCF, TFER
c----------------------------------------------------------------------
      include 'params.trc'
      include 'ctldat.trc'
      include 'qa.trc'

c --- Local string variables
      character*30 dot30
      character*132 filectl
      character*180 command

      data filectl/'TRAJCRD.INP'/
      data dot30/'..............................'/

c --- Report progress
      write(*,*)
      write(*,*)'TRAJCRD Module Running'
      write(*,*)'SETUP PHASE'

c --- Get date and time from system
      call DATETM(rdate,rtime,rcpu)

c --- Overwrite default control file name if provided on command line
      call COMLINE(filectl)

c --- Process instructions from the control file
      open(ictl,file=filectl,status='old')
      call RDCF

c --- Open list file
      open(ilst,file=filelst)

c --- Report initial information to the list file
c --- Write banner
      write(ilst,5) ver,level
5     format(///,26x,'TRAJCRD OUTPUT SUMMARY',/,19x,'VERSION:  ',A12,
     1       ' LEVEL:  ',A12///)

c --- Transfer TRAJCRD control file to list file
      write(ilst,'(a80/)')dot30//' Control File Image '//dot30
      call TFER(ictl,ilst)
      write(ilst,'(a80///)')dot30//' Control File Image '//dot30

      return
      end

c----------------------------------------------------------------------
      subroutine rdcf
c----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01     Level: 081223                   RDCF
c ---           D. Strimaitis
c
c --- PURPOSE:  Reads control file for TRAJCRD application
c
c --- INPUTS:
c
c
c --- OUTPUT:
c
c     Common block /CONTROL/
c            ntrjdat,itrjdat,lcfiles,
c            rnlat0t,relon0t,rnlat1t,rnlat2t,
c            iutmznt,feastt,fnortht,pmapt,datumt,datent,utmhemt,
c            clat0t,clon0t,clat1t,clat2t,
c            filelst,filetrj
c
c --- RDCF called by: (main)
c --- RDCF calls:     CASE320, RGET320, NOBLNK320, XTRACTLL
c----------------------------------------------------------------------
      include 'params.trc'
      include 'ctldat.trc'

c --- Local variables
      logical lerror
      character*1 xp,eq
      character*5 case
      character*16 dataset,dataver
      character*320 char320,blnk320,temp320

      data xp/'!'/
      data eq/'='/
      data lerror/.FALSE./

      do i=1,320
         blnk320(i:i)=' '
      enddo

c --- Condition default filenames
      filelst=ADJUSTL(filelst)

c --- Process first line of control file
      read(ictl,'(2a16)') dataset,dataver
      if(dataset.NE.'TRAJCRD.INP     ') then
         write(*,*)'Error in control file for TRAJCRD application'
         write(*,*)'Expected DATASET name: TRAJCRD.INP'
         write(*,*)'Found    DATASET name: ',dataset
         stop 'HALTED in RDCF'
      endif
      if(dataver.NE.'1.0             ') then
         write(*,*)'Error in control file for TRAJCRD application'
         write(*,*)'Expected Version: 1.0'
         write(*,*)'Found    VERSION: ',dataver
         stop 'HALTED in RDCF'
      endif

c --- Set initial values
c --- Local counters
      ntrj=0

c --- Read each record and extract information between !delimiters!
20    char320=blnk320
      read(ictl,'(a320)',end=21) char320

c --- Create upper-case version of line
      klast=LEN_TRIM(char320)
      call CASE320('UPPER',char320,klast)

c --- Position of first 2 delimiters
      ixp1=INDEX(char320,xp)
      if(ixp1.EQ.0) then
c ---    No delimiters, skip this line
         goto 20
      else
         i1=ixp1+1
         i2=klast
         ixp2=INDEX(char320(i1:i2),xp)
         if(ixp2.GT.0) ixp2=ixp1+ixp2
      endif

c --- Must have 2 delimiters
      if(ixp1.GT.ixp2) then
c ---    Problem
         write(*,*)'Error in control file for TRAJCRD application'
         write(*,*)'Single delimiter found on a line:'
         write(*,*)char320(1:klast)
         stop 'HALTED in RDCF'
      endif

c --- Relative position of equals sign within delimiters
      ieq=INDEX(char320(ixp1:ixp2),eq)

c --- Only line with 2 delimiters and no equal sign should be !END!
      if(ieq.EQ.0) then
         if(INDEX(char320,'END').NE.0) then
c ---       Skip this line
            goto 20
         else
c ---       Problem
            write(*,*)'Error in control file for TRAJCRD application'
            write(*,*)'Delimiter-pair found without an assignment:'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif
      endif

c --- Set range of assigned value
      ieq=ixp1+ieq-1
      i1=ieq+1
      i2=ixp2-1

c --- Go through list of variables
c --------------------------------

c --- Number of input trajectory files
      if(INDEX(char320,'NTRJDAT').NE.0) then
         read(char320(i1:i2),'(i)') ntrjdat

c --- List file name
      elseif(INDEX(char320,'FILELST').NE.0) then
         read(char320(i1:i2),'(a)') filelst
         filelst=ADJUSTL(filelst)

c --- Trajectory file name
      elseif(INDEX(char320,'FILETRJ').NE.0) then
         ntrj=ntrj+1
         if(ntrj.GT.mxfil) then
            write(*,*)'Error in control file for TRAJCRD application'
            write(*,*)'Too many met files found: ',ntrj
            write(*,*)'Increase MXFIL and recompile'
            stop 'HALTED in RDCF'
         endif
         read(char320(i1:i2),'(a)') filetrj(ntrj)
         filetrj(ntrj)=ADJUSTL(filetrj(ntrj))

c --- Case for filename
      elseif(INDEX(char320,'LCFILES').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         temp320=ADJUSTL(temp320)
         if(temp320(1:1).EQ.'T') then
            lcfiles=.TRUE.
         elseif(temp320(1:1).EQ.'F') then
            lcfiles=.FALSE.
         else
            write(*,*)'Error in control file for TRAJCRD application'
            write(*,*)'Expected LCFILES = T or F:'
            write(*,*)char320(1:klast)
            stop 'HALTED in RDCF'
         endif

c --- Coordinate definition
      elseif(INDEX(char320,'PMAP').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         pmapt=temp320(1:8)
      elseif(INDEX(char320,'FEAST').NE.0) then
         call RGET320(char320,i1,i2,feastt)
      elseif(INDEX(char320,'FNORTH').NE.0) then
         call RGET320(char320,i1,i2,fnortht)
      elseif(INDEX(char320,'IUTMZN').NE.0) then
         read(char320(i1:i2),'(i)') iutmznt
      elseif(INDEX(char320,'UTMHEM').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         utmhemt=temp320(1:4)
      elseif(INDEX(char320,'RLAT0').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         clat0t=temp320(1:16)
      elseif(INDEX(char320,'RLON0').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         clon0t=temp320(1:16)
      elseif(INDEX(char320,'XLAT1').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         clat1t=temp320(1:16)
      elseif(INDEX(char320,'XLAT2').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         clat2t=temp320(1:16)
      elseif(INDEX(char320,'DATUM').NE.0) then
         temp320=blnk320
         read(char320(i1:i2),'(a)') temp320
         call NOBLNK320(temp320)
         datumt=temp320(1:8)

c --- Found unexpected variable assignment
      else
         write(*,*)'Error in control file for TRAJCRD application'
         write(*,*)'Unexpected variable assignment found:'
         write(*,*)char320(1:klast)
         stop 'HALTED in RDCF'
      endif

      goto 20

21    continue

c --- Check number of files
      if(ntrj.NE.ntrjdat) then
         write(*,*)'Error in control file for TRAJCRD application'
         write(*,*)'Number of input files found    : ',ntrj
         write(*,*)'does not match number expected : ',ntrjdat
         stop 'HALTED in RDCF'
      endif

c --- Test information
      if(ntrjdat.LE.0) lerror=.TRUE.
      if(LERROR) then
         write(*,*)'Error in control file for TRAJCRD application'
         write(*,*)'Some required information is MISSING or INVALID'
         write(*,*)'ntrjdat > 0?         ',ntrjdat
         stop 'HALTED in RDCF'
      endif

c --- Apply filename case instruction
      if(LCFILES) then
         case='LOWER'
      else
         case='UPPER'
      endif


      temp320=blnk320
      klast=LEN_TRIM(filelst)
      temp320(1:klast)=filelst(1:klast)
      call CASE320(case,temp320,klast)
      filelst(1:klast)=temp320(1:klast)

      do k=1,ntrjdat
         temp320=blnk320
         klast=LEN_TRIM(filetrj(k))
         temp320(1:klast)=filetrj(k)(1:klast)
         call CASE320(case,temp320,klast)
         filetrj(k)(1:klast)=temp320(1:klast)
      enddo

c --- Process lat/lon character variables to extract content
      call XTRACTLL(ilst,'LAT ',clat1t,rnlat1t)
      call XTRACTLL(ilst,'LAT ',clat2t,rnlat2t)
      call XTRACTLL(ilst,'LAT ',clat0t,rnlat0t)
      call XTRACTLL(ilst,'LON ',clon0t,relon0t)

      return
      end

c----------------------------------------------------------------------
      subroutine comp
c----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01     Level: 081223                   COMP
c ---           D. Strimaitis
c
c --- PURPOSE:  Main computational component of program
c
c --- INPUTS:
c             none
c
c --- OUTPUT:
c             none
c
c --- COMP called by: (main)
c
c --- COMP calls:      PROCHD, GLOBE1, GLOBE
c----------------------------------------------------------------------
      include 'params.trc'
      include 'ctldat.trc'
      include 'trajhd.trc'

c --- Local Variables
      logical llinp, llout
      character*1 cma
      character*320 cinp320, blnk320
      character*32 cxy
c --- For coordinate transformations
      character*8 cmapi,cmapo
      character*12 caction
      character*4 c4hem
      real*8 vecti(9),vecto(9)

      data cma/','/

      do n=1,320
         blnk320(n:n)=' '
      enddo

c --- Report progress
      write(*,*)'PROCESSING PHASE'

c --- Set output projection
c --- Scale factor for Tangential TM projection
      tmsone=1.00000
      iutmo=iutmznt
      if(utmhemt.EQ.'S   ' .AND. iutmo.LT.900) iutmo=-iutmo
      cmapo=pmapt
      if(cmapo.EQ.'TTM     ') cmapo='TM      '

c --- Loop over trajectory files
c ------------------------------
      do n=1,ntrjdat
         nlen=LEN_TRIM(filetrj(n))
         open(itrj,file=filetrj(n)(1:nlen),status='old')
         open(iplt,file=filetrj(n)(1:nlen)//'.plt',status='unknown')

c ---    Read header into /TRAJHD/ and write header to plot file
         call PROCHD

c ---    Set input projection
         iutmi=iutmzn
         if(utmhem.EQ.'S   ' .AND. iutmi.LT.900) iutmi=-iutmi
         cmapi=pmap
         if(cmapi.EQ.'TTM     ') cmapi='TM      '
c ---    Set conversion vectors
         call GLOBE1(cmapi,iutmi,tmsone,rnlat1,rnlat2,
     &                  rnlat0,relon0,feast,fnorth,
     &                  cmapo,iutmo,tmsone,rnlat1t,rnlat2t,
     &                  rnlat0t,relon0t,feastt,fnortht,
     &                  caction,vecti,vecto)

c ---    Set projection flag category
         llinp=.FALSE.
         llout=.FALSE.
         if(pmap.EQ.'LL      ')  llinp=.TRUE.
         if(pmapt.EQ.'LL      ') llout=.TRUE.

c ---    Process data records (fixed format from TRAJ2D)
20       cinp320=blnk320
         read(itrj,'(a320)',end=999)cinp320
         ncinp=LEN_TRIM(cinp320)
         if(LLINP) then
c ---       Read longitude,latitude format string
            ncr=46
            read(cinp320,'(17x,2(f13.7,1x))') xin,yin
         else
c ---       Read x,y format string
            ncr=42
            read(cinp320,'(17x,2(f9.3,1x))') xin,yin
         endif
c ---    Convert map projection
         call GLOBE(ilst,caction,datum,vecti,datumt,vecto,
     &              xin,yin,xout,yout,izone,c4hem)
         if(LLOUT) then
c ---       Write longitude,latitude format record
            write(cxy,'(2(f13.7,a1),4x)') xout,cma,yout,cma
            write(iplt,'(a)') cinp320(1:17)//cxy(1:28)//
     &                        cinp320(ncr:ncinp)
         else
c ---       Write x,y format record
            write(cxy,'(2(f9.3,a1),12x)') xout,cma,yout,cma
            write(iplt,'(a)') cinp320(1:17)//cxy(1:24)//
     &                        cinp320(ncr:ncinp)
         endif

         goto 20

999      close(itrj)
         close(iplt)
      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine prochd
c----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01     Level: 081223                 PROCHD
c ---           D. Strimaitis
c
c --- PURPOSE:  Read header from trajectory file and write updated
c               header to plot file (same format)
c
c --- INPUTS:
c
c      common/QA/
c            ver,level
c
c     Common block /CONTROL/
c            rnlat0t,relon0t,rnlat1t,rnlat2t,
c            iutmznt,feastt,fnortht,pmapt,datumt,datent,utmhemt,
c            clat0t,clon0t,clat1t,clat2t
c
c
c
c     Parameters:
c           MXFIL, ITRJ, IPLT
c
c --- OUTPUT:
c      common/TRAJHD/
c             rnlat0,relon0,rnlat1,rnlat2,
c             iutmzn,feast,fnorth,pmap,datum,daten,utmhem,
c             xyunit,clat0,clon0,clat1,clat2,axbtz,
c             dataver,dataset,datamod
c
c
c --- PROCHD called by:  COMP
c --- PROCHD calls:      XTRACTLL
c----------------------------------------------------------------------
      include 'params.trc'
      include 'ctldat.trc'
      include 'qa.trc'
      include 'trajhd.trc'

c --- Local variables
      character*320 char320,blnk320
      character*16 dname,dver
      character*64 doc64

      do n=1,320
         blnk320(n:n)=' '
      enddo

c --- File ID record
      read(itrj,'(2a16,a64)') dname,dver,doc64
      write(iplt,'(2a16,a64)') dname,dver,doc64

c --- Comment block 1
      read(itrj,*) n1
      do n=1,n1
         read(itrj,*)
      enddo
      n1=1
      write(iplt,'(i1)') n1
      write(iplt,'(a)') 'Produced by TRAJCRD  Version: '//ver//
     &                  'Level: '//level

c --- Comment block 2
      read(itrj,*) n2
      write(iplt,'(i1)') n2
      do n=1,n2
         char320=blnk320
         read(itrj,'(a320)')char320
         nlen=LEN_TRIM(char320)
         write(iplt,'(a)') char320(1:nlen)
      enddo

c --- Input coordinate documentation (space and time)
      read(itrj,'(a8)') pmap
      if(pmap.EQ.'UTM     ') then
         read(itrj,'(i4,a4)') iutmzn,utmhem
      elseif(pmap.NE.'LL      ') then
         if(pmap.EQ.'LCC     ') then
            read(itrj,'(4a16)') clat0,clon0,clat1,clat2
         elseif(pmap.EQ.'PS      ') then
            read(itrj,'(4a16)') clat0,clon0,clat1
         else 
            read(itrj,'(4a16)') clat0,clon0
         endif
         if(pmap.EQ.'LCC     ' .or. 
     &      pmap.EQ.'LAZA    ' .or.
     &      pmap.EQ.'TTM     ') then
            read(itrj,'(2(1x,e14.7))') feast,fnorth
         endif
      endif
      read(itrj,'(a8,a12)') datum,daten
      read(itrj,'(a4)') xyunit
      read(itrj,'(a8)') axbtz

c --- Output coordinate documentation (space and time)
      write(iplt,'(a8)') pmapt
      if(pmapt.EQ.'LL      ') then
         write(iplt,'(a8,a12)') datumt,daten
         write(iplt,'(a4)') ' DEG'
         write(iplt,'(a8)') axbtz
      else
         if(pmapt.EQ.'UTM     ') then
            write(iplt,'(i4,a4)') iutmznt,utmhemt
         else
            if(pmapt.EQ.'LCC     ') then
               write(iplt,'(4a16)') clat0t,clon0t,clat1t,clat2t
            elseif(pmapt.EQ.'PS      ') then
               write(iplt,'(4a16)') clat0t,clon0t,clat1t
            else
               write(iplt,'(4a16)') clat0t,clon0t
            endif
            if(pmapt.EQ.'LCC     ' .OR. 
     &         pmapt.EQ.'LAZA    ' .OR.
     &         pmapt.EQ.'TTM     ') then
               write(iplt,'(2(1x,e14.7))') feastt,fnortht
            endif
         endif
         write(iplt,'(a8,a12)') datumt,daten
         write(iplt,'(a4)') '  KM'
         write(iplt,'(a8)') axbtz
      endif

c --- Comment block 3
      read(itrj,*) n3
      do n=1,n3
         read(itrj,*)
      enddo
      n3=3
      write(iplt,'(i1)') n3
      if(pmapt.EQ.'LL      ') then
         write(iplt,'(2a36)')'Trajectory Start           Location ',
     &                       '          Time Back    Current Time '
         write(iplt,'(2a36)')' YYYYJJJHH  SSSS     LON(DEG)      L',
     &                       'AT(DEG)     HOURS      JJJ HH  SSSS '
      else
         write(iplt,'(2a34)')'Trajectory Start        Location  ',
     &                       '        Time Back    Current Time '
         write(iplt,'(2a34)')' YYYYJJJHH  SSSS     XKM       YKM',
     &                       '          HOURS      JJJ HH  SSSS '
      endif
      write(iplt,*)

c --- Process lat/lon character variables to extract content
      call XTRACTLL(ilst,'LAT ',clat1,rnlat1)
      call XTRACTLL(ilst,'LAT ',clat2,rnlat2)
      call XTRACTLL(ilst,'LAT ',clat0,rnlat0)
      call XTRACTLL(ilst,'LON ',clon0,relon0)

      return
      end

c----------------------------------------------------------------------
      subroutine tfer(iinp,iout)
c----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01     Level: 081223                   TFER
c ---           D. Strimaitis
c
c --- PURPOSE:  Transfers one file image to another file
c
c --- INPUTS:
c             IINP - integer      - Unit number for input file
c             IOUT - integer      - Unit number for output file
c
c --- OUTPUT:
c
c
c --- TFER called by: (main)
c --- TFER calls:     none
c----------------------------------------------------------------------

c --- Local variables
      character*640 char640,blnk640
      do i=1,640
         blnk640(i:i)=' '
      enddo

c --- Back to start of file
      REWIND(iinp)

c --- Read and write each record
20    char640=blnk640
      read(iinp,'(a640)',end=21) char640
      klast=LEN_TRIM(char640)
      write(iout,'(a)') char640(1:klast)
      goto 20

21    close(iinp)

      return
      end

c----------------------------------------------------------------------
      subroutine fin
c----------------------------------------------------------------------
c
c --- TRAJCRD  Version: 1.01     Level: 081223                     FIN
c ---          D. Strimaitis
c
c --- PURPOSE:  Run termination routine -- compute runtime,
c               write last day processed
c
c --- INPUTS:  none
c
c --- OUTPUT:  none
c
c --- FIN called by:  (main)
c --- FIN calls:      DATETM, JULDAY, DELTT
c----------------------------------------------------------------------
c --- Include parameters
      include 'params.trc'
      include 'qa.trc'

      character*8 rtime2
      character*10 rdate2

      write(*,*)
      write(*,*)'TERMINATION PHASE'

c --- Get system date & time at end of run
      call DATETM(rdate2,rtime2,rcpu)

c --- Compute runtime
      read(rtime(1:2),10)ihr1
      read(rtime(4:5),10)imin1
      read(rtime(7:8),10)isec1
10    format(i2)
      t1=ihr1*3600.+imin1*60.+isec1

      read(rtime2(1:2),10)ihr2
      read(rtime2(4:5),10)imin2
      read(rtime2(7:8),10)isec2
      t2=ihr2*3600.+imin2*60.+isec2

      if(rdate.eq.rdate2)then
         delt=t2-t1
      else
         read(rdate(1:2),10)imo1
         read(rdate(4:5),10)iday1
         read(rdate(7:10),'(i4)')iyr1
         call julday(ilst,iyr1,imo1,iday1,ijul1)

         read(rdate2(1:2),10)imo2
         read(rdate2(4:5),10)iday2
         read(rdate2(7:10),'(i4)')iyr2
         call julday(ilst,iyr2,imo2,iday2,ijul2)

c ---    Compute no. hours from beg. of first hour of run to
c ---    ending hour of ending day of the run
         call deltt(iyr1,ijul1,ihr1,iyr2,ijul2,ihr2,idelhr)

c ---    Adjust for minutes and seconds
         delt=idelhr*3600.-imin1*60.-isec1+imin2*60.+isec2
      endif

c --- On the PC, the runtime and CPU time are the same
c --- (DATETM provides RCPU = 0.0 on the PC)
      if(rcpu.eq.0.0)rcpu=delt

c --- Report current date
      write(ilst,1402)rtime2,rdate2,delt,rcpu
1402  format(//2x,'End of run -- Clock time: ',a8/
     1         2x,'                    Date: ',a10//
     2         2x,'      Elapsed clock time: ',f10.1,' (seconds)'//
     3         2x,'                CPU time: ',f10.1,' (seconds)')

      return
      end

c----------------------------------------------------------------------
      subroutine case320(mode,c320,nlim)
c----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01     Level: 081223                CASE320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert all letters within a character string to 
c               either upper or lower case
c
c --- INPUTS:
c
c             MODE - character*5       - Output directive for either 
c                                        upper or lower case
c             C320 - character*320     - Input character string
c             NLIM - integer           - Length of string (characters)
c
c --- OUTPUT:
c
c             C320 - character*320     - Output character string with
c                                        uniform case
c
c --- CASE320 called by: (utility)
c --- CASE320 calls:      IACHAR, CHAR
c----------------------------------------------------------------------

      character*320 c320
      character*5 mode

      if(nlim.GT.320) then
         write(*,*)'ERROR in Subr. CASE320 - string is too long'
         write(*,*)'Maximum length  : 320'
         write(*,*)'Requested length: ',nlim
         stop 'Program HALTED'
      endif

      if(mode.EQ.'UPPER' .OR. mode.EQ.'upper') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.97 .AND. ia.LE.122) then
               ia=ia-32
               c320(k:k)=CHAR(ia)
            endif
         enddo

      elseif(mode.EQ.'LOWER' .OR. mode.EQ.'lower') then
         do k=1,nlim
            ia=IACHAR(c320(k:k))
            if(ia.GE.65 .AND. ia.LE.90) then
               ia=ia+32
               c320(k:k)=CHAR(ia)
            endif
         enddo
      else
         write(*,*)'ERROR in Subr. CASE320 - invalid output mode'
         write(*,*)'Expected UPPER or LOWER'
         write(*,*)'Found: ',mode
         stop 'Program HALTED'
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine noblnk320(c320)
c----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01     Level: 081223              NOBLNK320
c ---           D. Strimaitis
c
c --- PURPOSE:  Remove all blanks from string (left-justify)
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c
c --- OUTPUT:
c
c             C320 - character*320     - Output character string with
c
c --- NOBLNK320 called by: (utility)
c --- NOBLNK320 calls:      ADJUSTL, LEN_TRIM
c----------------------------------------------------------------------

      character*320 c320,d320

c --- Blank out work string
      do i=1,320
         d320(i:i)=' '
      enddo

c --- Remove leading blanks
      c320=ADJUSTL(c320)
      last=LEN_TRIM(c320)

c --- Transfer non-blanks into work string
      n=0
      do i=1,last
         if(c320(i:i).NE.' ') then
            n=n+1
            d320(n:n)=c320(i:i)
         endif
      enddo

c --- Pass work string
      c320=d320

      return
      end

c----------------------------------------------------------------------
      subroutine rget320(c320,i1,i2,x)
c----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01     Level: 081223                RGET320
c ---           D. Strimaitis
c
c --- PURPOSE:  Convert character string to a real.  This sub is needed
c               when there is no decimal point.  The LF95 compiler
c               does not do the internal read conversion without the '.'
c               Also, determine when exponential notation is present.
c
c --- INPUTS:
c
c             C320 - character*320     - Input character string
c               i1 - integer           - First character of substring
c               i2 - integer           - Last character of substring
c
c --- OUTPUT:
c
c                X - real              - Output real
c
c --- RGET320 called by: (utility)
c --- RGET320 calls:      none
c----------------------------------------------------------------------
      character*320 c320
      character*20 c20, blnk20
      data blnk20/'                    '/

      if(i1.GT.i2) return

      c20=blnk20
      c20=c320(i1:i2)

c --- Left justify field
      c20=ADJUSTL(c20)

c --- Test for exponential notation
      ie=MAX(INDEX(c20,'e'),INDEX(c20,'E'))

      if(ie.EQ.0) then
c ---    Simple real number
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            ipt=LEN_TRIM(c20)+1
            c20(ipt:ipt)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(f)') x

      else
c ---    Exponential notation
         ipt=INDEX(c20,'.')
         if(ipt.EQ.0) then
c ---       Decimal point NOT found
            do k=20,ie+1,-1
               km1=k-1
               c20(k:k)=c20(km1:km1)
            enddo
            c20(ie:ie)='.'
         endif
c ---    Decimal point in place: internal read OK
         read(c20,'(e)') x

      endif

      return
      end

c----------------------------------------------------------------------
      subroutine xtractll(io,type,clatlon,rlatlon)
c----------------------------------------------------------------------
c
c --- TRAJCRD   Version: 1.01     Level: 081223               XTRACTLL
c               D. Strimaitis
c
c --- PURPOSE:  Extract the real latitude or longitude from a character
c               string that contains the N/S or E/W convention
c               character, and express result as either North Latitude
c               or East Longitude
c
c --- INPUTS:
c               IO - integer    - Unit number for list file output
c             TYPE - char*4     - LAT or LON
c          CLATLON - char*16    - Latitude or longitude (degrees), with
c                                 1 character that denotes convention
c                                 (e.g. 'N  45.222' or  '-35.999s')
c
c --- OUTPUT:
c          RLATLON - real       - North Latitude or East Longitude
c                                 (degrees)
c
c --- XTRACTLL called by: (utility)
c --- XTRACTLL calls:     NOBLNK320, CASE320, RGET320
c----------------------------------------------------------------------
      character*320 c320,blnk320
      character*16 clatlon, clatlon2
      character*4 type
      logical ltype

      ltype=.FALSE.

c --- Place string into work string 
      c320(1:16)=clatlon
      do i=17,320
         c320(i:i)=' '
      enddo

c --- Remove blanks enforce upper case
      call NOBLNK320(c320)
      klast=LEN_TRIM(c320)
      call CASE320('UPPER',c320,klast)

c --- Was valid type provided?
      if(type.NE.'LAT ' .AND. type.NE.'LON ') then
         write(io,*) 'XTRACTLL:  FATAL ERROR reported when ',
     &               'extracting Latitude/Longitude'
         write(io,*) 'Invalid type:  ',type
         write(io,*) 'Expected LAT or LON'
         write(*,*)
         stop 'Halted in XTRACTLL -- see list file'
      endif

c --- Interpret valid convention character (N,S,E,W)
      nchar=0
      ichar=0
      ilat=0
      ilon=0
   
      do i=1,klast
         if(c320(i:i).EQ.'N') then
            ilat=1
            ichar=i
            nchar=nchar+1
         elseif(c320(i:i).EQ.'S') then
            ilat=2
            ichar=i
            nchar=nchar+1
         elseif(c320(i:i).EQ.'W') then
            ilon=1
            ichar=i
            nchar=nchar+1
         elseif(c320(i:i).EQ.'E') then
            ilon=2
            ichar=i
            nchar=nchar+1
         endif
      enddo

c --- Was 1 valid character found?
      if(nchar.NE.1) then
         write(io,*) 'XTRACTLL:  FATAL ERROR reported when ',
     &               'extracting Latitude/Longitude'
         write(io,*) 'N,S,E,W character is missing or repeated'
         write(io,*) 'Lat/Lon = ',clatlon
         write(*,*)
         stop 'Halted in XTRACTLL -- see list file'
      endif

c --- Was valid character the right type?
      if(type.EQ.'LAT ' .AND. ilat.EQ.0) ltype=.TRUE.
      if(type.EQ.'LON ' .AND. ilon.EQ.0) ltype=.TRUE.
      if(LTYPE) then
         write(io,*) 'XTRACTLL:  FATAL ERROR reported when ',
     &               'extracting Latitude/Longitude'
         write(io,*) 'N,S,E,W character does not match type'
         write(io,*) 'Lat/Lon = ',clatlon
         write(io,*) 'type    = ',type
         write(*,*)
         stop 'Halted in XTRACTLL -- see list file'
      endif

c --- Remove character from string
      do i=ichar,klast
         ip1=i+1
         c320(i:i)=c320(ip1:ip1)
      enddo

c --- Extract real degrees
      call RGET320(c320,1,klast,rlatlon)

c --- Convert to either N. Lat. or E. Lon., if needed
      if(ilat.EQ.2) then
         rlatlon=-rlatlon
      elseif(ilon.EQ.1) then
         rlatlon=-rlatlon
      endif

c --- Condition longitude to be -180 to +180
      if(ilon.GT.0) then
         if(rlatlon.GT.180.) then
            rlatlon=rlatlon-360.
         elseif(rlatlon.LT.-180.) then
            rlatlon=rlatlon+360.
         endif
      endif

      return
      end
